#!/bin/bash
# Generate device certificate

usage="Usage info: ./gen_certificate.sh [-h HOSTNAME] [-s START_DATE] IP_ADDRESS"

# Check arguments
if [ $# -lt 1 ]
then
    echo $usage
    exit 1;
fi

while getopts "h:s:" opt; do
 	case "$opt" in
		h) HNAME=${OPTARG};;
		s) START_DATE=${OPTARG};;
		*) echo $usage:exit 1;;
	esac
done

if [ $OPTIND -eq 5 ] && [ $# -ge 5 ]
then
	IP=$5
elif [ $OPTIND -eq 3 ] && [ $# -ge 3 ]
then
	IP=$3
elif [ $OPTIND -eq 1 ] && [ $# -ge 1 ]
then
	IP=$1
else
	echo $usage
	exit 1
fi

# Default values
if [ -z "$HNAME" ]
then
	HNAME=$IP
fi

if [ -z "$START_DATE" ]
then
	START_DATE=`date +%Y%m%d000000Z`
fi

CERT="certificate"
CA="rootCA"
CA_SUBJECT=$(openssl x509 -noout -subject -in $CA.crt | sed -n '/^subject/s/^subject= //p')
SUBJECT=${CA_SUBJECT/CN=*/CN=$HNAME}

# Set required files
if ! [ -s "index.txt" ]
then
	rm -rf newcerts
	mkdir newcerts
	echo -n > index.txt
	echo '01' > serial
fi

# Generate the certificate signing request
openssl req -new -subj "$SUBJECT" -key $CERT.key -out $CERT.csr

# Check signing request
csrCheck=$(openssl req -noout -verify -in $CERT.csr 2>&1)
if [[ $csrCheck != "verify OK" ]]
then
  echo "CSR file not correct"
  exit 1;
fi

# Copy template openssl_tmpl.conf to openssl.conf
cp openssl_tmpl.conf openssl.conf

# Edit openssl.conf with the current IP
echo "IP.1 = $IP" >> openssl.conf

# Create CRT
openssl ca -batch -config ca.conf -keyfile $CA.key -cert $CA.crt -startdate $START_DATE -in $CERT.csr -out $CERT.crt -extfile openssl.conf

# Check certificate
crtCheck=$(openssl verify -CAfile $CA.crt $CERT.crt)
echo $crtCheck
if [[ $crtCheck != "$CERT.crt: OK" ]]
then
  echo "CRT file not correct"
  exit 1;
fi

# Concatenate crt and key in a .pem file that could be used by lighttpd
cat $CERT.key $CERT.crt > $CERT.pem

# Delete temp files
rm -f openssl.conf

# Restart lighttpd
/etc/init.d/lighttpd restart

echo "Certificate generation OK"
exit 0;
